@@ -3,7 +3,7 @@
 #' @importFrom stats coef
 
 print.Tenv <- function(x, ...){
-  cat("\nCall:\n", paste(deparse(x$call), sep = "\n", collapse = "\n"),
+  cat("Call:\n", paste(deparse(x$call), sep = "\n", collapse = "\n"),
       "\n\n", sep = "")
   cat("Coefficients:\n")
   cf <- coef(x)
